/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.client.rfs;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import fr.cnrs.mri.macro.io.IOSettings;

public class RFSClientApplication extends Observable implements Observer {

	public enum Aspect {
		PORT, SERVER, PING_FAILED, PING_OK, STATUS, CONNECTION_FAILURE, CONNECTION};
	private String status;
	private int port;
	private String server;
	private RemoteFilesystemView remoteFilesystemView;
	private RFSClientApplicationView view;
	
	static public List<String> paths = new ArrayList<String>();
	
	public RFSClientApplication() {
		IOSettings.getInstance().addObserver(this);
	}
	
	void changed(Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}
	
	public String getStatus() {
		return this.status;
	}

	public void setStatus(String status) {
		this.status = status;
		this.changed(Aspect.STATUS);
	}
	
	public int getPort() {
		return this.port;
	}

	public void setPort(String aPort) {
		int port = this.port;
		try {
			port = Integer.parseInt(aPort);
			this.port = port;
		}  catch (NumberFormatException e) {
			port = this.port;
		}
		this.changed(Aspect.PORT);
	}
	
	public String getServer() {
		return server;
	}

	public void setServer(String text) {
		this.server = text;
		this.changed(Aspect.SERVER);
	}

	public void ping() {
		this.setStatus("Trying ping...");
		RemoteFilesystemClient client =  new RemoteFilesystemClient(this.getServer(), Integer.toString(this.getPort()));
		String answer = client.ping();
		client.close();
		if (answer!=null && answer.equals("pong")) this.changed(Aspect.PING_OK); 
		else this.changed(Aspect.PING_FAILED);
	}

	@Override
	public void update(Observable sender, Object anAspect) {
		if (sender!=IOSettings.getInstance()) return; 
		this.changed(Aspect.CONNECTION);
	}

	public boolean isRunning() {
		return IOSettings.getFilesystemView()!=null;
	}

	public void stop() {
		remoteFilesystemView.close();
		IOSettings.setFilesystemView(null);
	}

	public void start() {
		remoteFilesystemView = new RemoteFilesystemView(this.getServer(), 
		  							   Integer.toString(this.getPort()));
		IOSettings.setFilesystemView(remoteFilesystemView);
	}

	public RemoteFilesystemView getRemoteFilesystemView() {
		return remoteFilesystemView;
	}

	public void show() {
		this.getView().setVisible(true);
	}

	private RFSClientApplicationView getView() {
		if (this.view == null) this.view = new RFSClientApplicationView(this);
		return view;
	}

}
