/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.client.rfs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.Observable;
import java.util.Observer;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTextField;
import fr.cnrs.mri.remoteij.client.rfs.RFSClientApplication.Aspect;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

public class RFSClientApplicationView extends JFrame implements Observer {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JPanel jPanel = null;
	private JTextField serverTextField = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField portTextField = null;
	private JButton testButton = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel6 = null;
	private JButton runButton = null;
	private RFSClientApplication model;
	private JPanel jPanel2 = null;
	private JLabel jLabel2 = null;
	private JTextField statusTextField = null;
	private JLabel jLabel3 = null;
	private JTextField infoTextField = null;
	
	@Override
	public void update(Observable sender, Object aspect) {
		if (aspect.equals(Aspect.SERVER)) this.handleServerChanged();
		if (aspect.equals(Aspect.PORT)) this.handlePortChanged();
		if (aspect.equals(Aspect.PING_FAILED)) this.handlePingFailed();
		if (aspect.equals(Aspect.PING_OK)) this.handlePingOK();
		if (aspect.equals(Aspect.STATUS)) this.handleInfoChanged();
		if (aspect.equals(Aspect.CONNECTION_FAILURE)) this.handleConnectionFailure();
		if (aspect.equals(Aspect.CONNECTION)) this.handleStatusChanged();
	}

	private void handleStatusChanged() {
		String status = "";
		RemoteFilesystemView filesystemView = model.getRemoteFilesystemView();
		if (model.isRunning()) {
			status = "connected to " + filesystemView.getHost() + ":" + filesystemView.getPort();
			getRunButton().setText("stop");
		}
		else {
			status = "not connected";
			getRunButton().setText("start");
		}
		this.getStatusTextField().setText(status);
	}

	private void handleConnectionFailure() {
		this.getInfoTextField().setText("Connection failure");
	}

	private void handleInfoChanged() {
		this.getInfoTextField().setText(model.getStatus());
	}

	private void handlePingOK() {
		this.getInfoTextField().setText("Ping successful");
	}

	private void handlePingFailed() {
		this.getInfoTextField().setText("Ping failed");
	}

	private void handlePortChanged() {
		this.getPortTextField().setText(Integer.toString(model.getPort()));
	}

	private void handleServerChanged() {
		this.getServerTextField().setText(model.getServer());
	}

	/**
	 * This is the default constructor
	 */
	public RFSClientApplicationView() {
		super();
		this.model = new RFSClientApplication();
		model.addObserver(this);
		initialize();
	}

	public RFSClientApplicationView(RFSClientApplication model) {
		super();
		this.model = model;
		model.addObserver(this);
		initialize();
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(423, 148);
		this.setResizable(false);
		this.setMaximumSize(new Dimension(2147483647, 148));
		this.setContentPane(getJContentPane());
		this.setTitle("Remote Filesystem Client");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BoxLayout(getJContentPane(), BoxLayout.Y_AXIS));
			jContentPane.add(getJPanel1(), null);
			jContentPane.add(getJPanel2(), null);
			jContentPane.add(getJPanel6(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jLabel1 = new JLabel();
			jLabel1.setText("port:");
			jLabel = new JLabel();
			jLabel.setText("server:");
			jPanel = new JPanel();
			jPanel.setLayout(new FlowLayout());
			jPanel.setMaximumSize(new Dimension(32767, 35));
			jPanel.add(jLabel, null);
			jPanel.add(getServerTextField(), null);
			jPanel.add(jLabel1, null);
			jPanel.add(getPortTextField(), null);
			jPanel.add(getTestButton(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes serverTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getServerTextField() {
		if (serverTextField == null) {
			serverTextField = new JTextField();
			serverTextField.setPreferredSize(new Dimension(128, 19));
			serverTextField.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					updateServer();
				}
			});
			serverTextField.addFocusListener(new java.awt.event.FocusAdapter() {
				public void focusLost(java.awt.event.FocusEvent e) {
					updateServer();
				}
			});
		}
		return serverTextField;
	}

	protected void updateServer() {
		this.model.setServer(this.getServerTextField().getText());
	}

	/**
	 * This method initializes portTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPortTextField() {
		if (portTextField == null) {
			portTextField = new JTextField();
			portTextField.setPreferredSize(new Dimension(48, 19));
			portTextField.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					updatePort();
				}
			});
			portTextField.addFocusListener(new java.awt.event.FocusAdapter() {
				public void focusLost(java.awt.event.FocusEvent e) {
					updatePort();
				}
			});
		}
		return portTextField;
	}

	protected void updatePort() {
		this.model.setPort(this.getPortTextField().getText());
	}

	/**
	 * This method initializes testButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getTestButton() {
		if (testButton == null) {
			testButton = new JButton();
			testButton.setText("test");
			testButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					model.ping();
				}
			});
		}
		return testButton;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.setLayout(new BoxLayout(getJPanel1(), BoxLayout.X_AXIS));
			jPanel1.add(getJPanel(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel6	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel6() {
		if (jPanel6 == null) {
			jPanel6 = new JPanel();
			jPanel6.setLayout(new BoxLayout(getJPanel6(), BoxLayout.X_AXIS));
			jPanel6.add(getRunButton(), null);
		}
		return jPanel6;
	}

	/**
	 * This method initializes runButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getRunButton() {
		if (runButton == null) {
			runButton = new JButton();
			runButton.setText("start");
			runButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (model.isRunning()) model.stop();
					else model.start();
				}
			});
		}
		return runButton;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.ipadx = 206;
			gridBagConstraints3.ipady = 4;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.insets = new Insets(0, 0, 0, 1);
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.ipadx = 0;
			gridBagConstraints2.ipady = 8;
			gridBagConstraints2.anchor = GridBagConstraints.EAST;
			gridBagConstraints2.gridy = 1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.ipadx = 206;
			gridBagConstraints1.ipady = 4;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.anchor = GridBagConstraints.CENTER;
			gridBagConstraints1.insets = new Insets(0, 0, 0, 1);
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.ipadx = 0;
			gridBagConstraints.ipady = 8;
			gridBagConstraints.gridwidth = 1;
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.gridy = 0;
			jLabel3 = new JLabel();
			jLabel3.setText("info:");
			jLabel3.setName("jLabel3");
			jLabel2 = new JLabel();
			jLabel2.setText("status:");
			jLabel2.setName("jLabel2");
			GridLayout gridLayout = new GridLayout();
			gridLayout.setRows(2);
			gridLayout.setColumns(2);
			jPanel2 = new JPanel();
			jPanel2.setLayout(new GridBagLayout());
			jPanel2.setMaximumSize(new Dimension(2147483647, 46));
			jPanel2.add(jLabel2, gridBagConstraints);
			jPanel2.add(getStatusTextField(), gridBagConstraints1);
			jPanel2.add(jLabel3, gridBagConstraints2);
			jPanel2.add(getInfoTextField(), gridBagConstraints3);
		}
		return jPanel2;
	}

	/**
	 * This method initializes statusTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getStatusTextField() {
		if (statusTextField == null) {
			statusTextField = new JTextField();
			statusTextField.setName("jTextField");
		}
		return statusTextField;
	}

	/**
	 * This method initializes infoTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getInfoTextField() {
		if (infoTextField == null) {
			infoTextField = new JTextField();
			infoTextField.setName("jTextField1");
		}
		return infoTextField;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
