/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.client.rmr;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JFrame;
import java.awt.FlowLayout;
import javax.swing.JComboBox;
import java.awt.Dimension;
import java.awt.event.ItemEvent;

import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import fr.cnrs.mri.remoteij.client.rmr.RMRClientApplication.Aspect;

public class RMRClientApplicationView extends JFrame implements Observer {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JPanel jPanel = null;
	private JTextField serverTextField = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField portTextField = null;
	private JButton testButton = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel2 = null;
	private JComboBox<Object> examplesComboBox = null;
	private JLabel jLabel2 = null;
	private JScrollPane jScrollPane = null;
	private JTextArea macroTextArea = null;
	private JPanel jPanel3 = null;
	private JPanel jPanel4 = null;
	private JLabel jLabel3 = null;
	private JTextField resultTextField = null;
	private JPanel jPanel5 = null;
	private JLabel jLabel4 = null;
	private JTextField statusTextField = null;
	private JPanel jPanel6 = null;
	private JButton runButton = null;
	private RMRClientApplication model;
	private JButton jButton = null;
	
	@Override
	public void update(Observable sender, Object aspect) {
		if (aspect.equals(Aspect.SERVER)) this.handleServerChanged();
		if (aspect.equals(Aspect.PORT)) this.handlePortChanged();
		if (aspect.equals(Aspect.PING_FAILED)) this.handlePingFailed();
		if (aspect.equals(Aspect.PING_OK)) this.handlePingOK();
		if (aspect.equals(Aspect.STATUS)) this.handleStatusChanged();
		if (aspect.equals(Aspect.CONNECTION_FAILURE)) this.handleConnectionFailure();
		if (aspect.equals(Aspect.RESULT_CHANGED)) this.handleResultChanged();
	}

	private void handleResultChanged() {
		this.getResultTextField().setText(model.getResult());
		this.getStatusTextField().setText("Macro successfully executed");
	}

	private void handleConnectionFailure() {
		this.getResultTextField().setText("");
		this.getStatusTextField().setText("Connection failure");
	}

	private void handleStatusChanged() {
		this.getStatusTextField().setText(model.getStatus());
	}

	private void handlePingOK() {
		this.getResultTextField().setText("Ping succeded");
		this.getStatusTextField().setText("Ping finished");
	}

	private void handlePingFailed() {
		this.getResultTextField().setText("Ping failed");
		this.getStatusTextField().setText("Ping finished");
	}

	private void handlePortChanged() {
		this.getPortTextField().setText(Integer.toString(model.getPort()));
	}

	private void handleServerChanged() {
		this.getServerTextField().setText(model.getServer());
	}

	/**
	 * This is the default constructor
	 */
	public RMRClientApplicationView() {
		super();
		this.model = new RMRClientApplication();
		model.addObserver(this);
		initialize();
	}

	public RMRClientApplicationView(RMRClientApplication model) {
		super();
		this.model = model;
		model.addObserver(this);
		initialize();
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(425, 308);
		this.setContentPane(getJContentPane());
		this.setTitle("Remote Macro Runner");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel1(), BorderLayout.NORTH);
			jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
			jContentPane.add(getJPanel3(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jLabel1 = new JLabel();
			jLabel1.setText("port:");
			jLabel = new JLabel();
			jLabel.setText("server:");
			jPanel = new JPanel();
			jPanel.setLayout(new FlowLayout());
			jPanel.add(jLabel, null);
			jPanel.add(getServerTextField(), null);
			jPanel.add(jLabel1, null);
			jPanel.add(getPortTextField(), null);
			jPanel.add(getTestButton(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes serverTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getServerTextField() {
		if (serverTextField == null) {
			serverTextField = new JTextField();
			serverTextField.setPreferredSize(new Dimension(128, 19));
			serverTextField.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					updateServer();
				}
			});
			serverTextField.addFocusListener(new java.awt.event.FocusAdapter() {
				public void focusLost(java.awt.event.FocusEvent e) {
					updateServer();
				}
			});
		}
		return serverTextField;
	}

	protected void updateServer() {
		this.model.setServer(this.getServerTextField().getText());
	}

	/**
	 * This method initializes portTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPortTextField() {
		if (portTextField == null) {
			portTextField = new JTextField();
			portTextField.setPreferredSize(new Dimension(48, 19));
			portTextField.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					updatePort();
				}
			});
			portTextField.addFocusListener(new java.awt.event.FocusAdapter() {
				public void focusLost(java.awt.event.FocusEvent e) {
					updatePort();
				}
			});
		}
		return portTextField;
	}

	protected void updatePort() {
		this.model.setPort(this.getPortTextField().getText());
	}

	/**
	 * This method initializes testButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getTestButton() {
		if (testButton == null) {
			testButton = new JButton();
			testButton.setText("test");
			testButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getResultTextField().setText("");
					getStatusTextField().setText("Trying ping...");
					model.ping();
				}
			});
		}
		return testButton;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.setLayout(new BoxLayout(getJPanel1(), BoxLayout.Y_AXIS));
			jPanel1.add(getJPanel(), null);
			jPanel1.add(getJPanel2(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jLabel2 = new JLabel();
			jLabel2.setText("examples:");
			jPanel2 = new JPanel();
			jPanel2.setLayout(new FlowLayout());
			jPanel2.add(jLabel2, null);
			jPanel2.add(getExamplesComboBox(), null);
			jPanel2.add(getJButton(), null);
		}
		return jPanel2;
	}

	/**
	 * This method initializes examplesComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox<?> getExamplesComboBox() {
		if (examplesComboBox == null) {
			examplesComboBox = new JComboBox<Object>(model.examplesNames());
			examplesComboBox.setPreferredSize(new Dimension(256, 24));
			examplesComboBox.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					if (e.getStateChange()!=ItemEvent.SELECTED) return;
					model.setExampleMacroName((String)getExamplesComboBox().getSelectedItem());
				}
			});
		}
		return examplesComboBox;
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getMacroTextArea());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes macroTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getMacroTextArea() {
		if (macroTextArea == null) {
			macroTextArea = new JTextArea();
		}
		return macroTextArea;
	}

	/**
	 * This method initializes jPanel3	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new JPanel();
			jPanel3.setLayout(new BoxLayout(getJPanel3(), BoxLayout.Y_AXIS));
			jPanel3.add(getJPanel4(), null);
			jPanel3.add(getJPanel5(), null);
			jPanel3.add(getJPanel6(), null);
		}
		return jPanel3;
	}

	/**
	 * This method initializes jPanel4	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			jLabel3 = new JLabel();
			jLabel3.setText("result: ");
			jPanel4 = new JPanel();
			jPanel4.setLayout(new BorderLayout());
			jPanel4.add(jLabel3, BorderLayout.WEST);
			jPanel4.add(getResultTextField(), BorderLayout.CENTER);
		}
		return jPanel4;
	}

	/**
	 * This method initializes resultTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getResultTextField() {
		if (resultTextField == null) {
			resultTextField = new JTextField();
		}
		return resultTextField;
	}

	/**
	 * This method initializes jPanel5	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel5() {
		if (jPanel5 == null) {
			jLabel4 = new JLabel();
			jLabel4.setText("status:");
			jPanel5 = new JPanel();
			jPanel5.setLayout(new BorderLayout());
			jPanel5.add(jLabel4, BorderLayout.WEST);
			jPanel5.add(getStatusTextField(), BorderLayout.CENTER);
		}
		return jPanel5;
	}

	/**
	 * This method initializes statusTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getStatusTextField() {
		if (statusTextField == null) {
			statusTextField = new JTextField();
		}
		return statusTextField;
	}

	/**
	 * This method initializes jPanel6	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel6() {
		if (jPanel6 == null) {
			jPanel6 = new JPanel();
			jPanel6.setLayout(new BoxLayout(getJPanel6(), BoxLayout.X_AXIS));
			jPanel6.add(getRunButton(), null);
		}
		return jPanel6;
	}

	/**
	 * This method initializes runButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getRunButton() {
		if (runButton == null) {
			runButton = new JButton();
			runButton.setText("run");
			runButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String macro = getMacroTextArea().getText();
					model.runMacro(macro);
				}
			});
		}
		return runButton;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText("add");
			jButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getMacroTextArea().setText(getMacroTextArea().getText()  + "\n" + model.getExampleMacro());
				}
			});
		}
		return jButton;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
