/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.queueManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.logging.Level;
import fr.cnrs.mri.remoteij.server.rmr.RMRServerConfig;
import fr.cnrs.mri.server.ServerLink;
import fr.cnrs.mri.sijame.messages.MessageQueue;
import fr.cnrs.mri.util.logging.LoggingUtil;

public class JobDispatcher implements Runnable {

	private static final String SERVERS_FILENAME = "servers.list";
	private MessageQueue requestQueue;
	private MessageQueue runningQueue;
	private boolean shallStop = false;
	private List<ServerLink> servers;

	public JobDispatcher(MessageQueue requestQueue, MessageQueue runningQueue) {
		this.requestQueue = requestQueue;
		this.runningQueue = runningQueue;
	}

	@Override
	public void run() {
		while(!shallStop) {
			
		}
	}

	public void stop() {
		this.shallStop = true;
	}

	public void addServer(ServerLink aServer) {
		if (!this.getServers().contains(aServer)) this.getServers().add(aServer);
	}

	public List<ServerLink> getServers() {
		if (this.servers == null) this.servers = new ArrayList<ServerLink>();
		return this.servers;
	}
	
	public boolean readServers() {
		boolean result = true;
		if (!new File(this.getServersFilename()).exists()) return false;
		    Scanner scanner = null;
			try {
				scanner = new Scanner(new FileInputStream(this.getServersFilename()));
				while (scanner.hasNextLine()){
		    	  String line = scanner.nextLine().trim();
		    	  if (line.isEmpty() || line.startsWith("//") || line.startsWith("#")) continue;
		    	  String[] parts = line.split(":");
		    	  String server = parts[0];
		    	  int port = RMRServerConfig.getCurrent().getDefaultPort();
		    	  if (parts.length>1) port = Integer.parseInt(parts[1]);
		    	  new ServerLink(server, port);
		      }
			} catch (FileNotFoundException e) {
				result = false;
				LoggingUtil.getLoggerFor(this).log(Level.SEVERE, "Cannot read server-list", e);
			}
		    finally{
		      if (scanner!=null) scanner.close();
		    }
		    return result;
	}

	public String getServersFilename() {
		return SERVERS_FILENAME;
	}
}
