package tests.fr.cnrs.mri.remoteij.server.rfs;

import static org.junit.Assert.*;

import java.io.File;

import org.junit.Before;
import org.junit.Test;

import fr.cnrs.mri.remoteij.server.rfs.RFSServerConfig;

public class RFSServerConfigTest {

	@Before
	public void setUp() throws Exception {
		new File(RFSServerConfig.getCurrent().getFilename()).delete();
		RFSServerConfig.reset();
	}

	@Test
	public void testGetCurrent() {
		assertNotNull(RFSServerConfig.getCurrent());
	}

	@Test
	public void testGetPort() {
		RFSServerConfig config = RFSServerConfig.getCurrent();
		assertEquals(config.getPort(), 4000);
		config.setPort(5000);
		assertEquals(config.getPort(), 5000);
		config.load();
		assertEquals(config.getPort(), 5000);
		RFSServerConfig otherConfig = new RFSServerConfig();
		assertEquals(otherConfig.getPort(), 5000);
		config.setPort(4000);
		config.load();
		assertEquals(config.getPort(), 4000);
	}
	
	@Test
	public void testGetFileserverRoot() {
		RFSServerConfig config = RFSServerConfig.getCurrent();
		assertEquals("/media/donnees/mri/projects/ImageJMS/fileserver/", config.getFileserverRoot());
		config.setFileserverRoot("E:/data/fileserver/");
		assertEquals(config.getFileserverRoot(), "E:/data/fileserver/");
		config.load();
		assertEquals(config.getFileserverRoot(), "E:/data/fileserver/");
		RFSServerConfig otherConfig = new RFSServerConfig();
		assertEquals(otherConfig.getFileserverRoot(), "E:/data/fileserver/");
	}


	@Test
	public void testGetUseUserFolders() {
		RFSServerConfig config = RFSServerConfig.getCurrent();
		assertTrue(config.getUseUserFolders());
		config.setUseUserFolders(false);
		assertFalse(config.getUseUserFolders());
		config.load();
		assertFalse(config.getUseUserFolders());
		RFSServerConfig otherConfig = new RFSServerConfig();
		assertFalse(otherConfig.getUseUserFolders());
		config.setUseUserFolders(true);
		assertTrue(config.getUseUserFolders());
	}

}
