package tests.fr.cnrs.mri.sijame.messages;

import static org.junit.Assert.*;

import java.io.File;
import java.util.Observable;
import java.util.Observer;
import org.junit.Test;
import fr.cnrs.mri.sijame.messages.Message;
import fr.cnrs.mri.sijame.messages.MessageQueue;
import fr.cnrs.mri.sijame.messages.RequestMessage;

public class MessageQueueTest implements Observer {
	
	private Message message;

	@Test
	public void testPush() {
		Message firstMessage = new RequestMessage("first message");
		Message secondMessage = new RequestMessage("second message");
		MessageQueue queue = new MessageQueue();
		queue.addObserver(this);
		assertTrue(queue.isEmpty());
		queue.push(firstMessage);
		assertEquals(firstMessage, this.message);
		assertFalse(queue.isEmpty());
		assertTrue(queue.size()==1);
		assertTrue(queue.front()==firstMessage);
		queue.push(secondMessage);
		assertEquals(firstMessage, this.message);
		assertTrue(queue.size()==2);
		assertTrue(queue.front()==firstMessage);
		queue.deleteObserver(this);
	}

	@Test
	public void testIsEmpty() {
		Message firstMessage = new RequestMessage("first message");
		MessageQueue queue = new MessageQueue();
		assertTrue(queue.isEmpty());
		queue.push(firstMessage);
		assertFalse(queue.isEmpty());
		queue.pop();
		assertTrue(queue.isEmpty());
	}
	
	@Test
	public void testSize() {
		Message firstMessage = new RequestMessage("first message");
		Message secondMessage = new RequestMessage("second message");
		MessageQueue queue = new MessageQueue();
		assertEquals(0, queue.size());
		queue.push(firstMessage);
		assertEquals(1, queue.size());
		queue.push(secondMessage);
		assertEquals(2, queue.size());
		queue.pop();
		assertEquals(1, queue.size());
		queue.pop();
		assertEquals(0, queue.size());
	}
	
	@Test
	public void testFront() {
		Message firstMessage = new RequestMessage("first message");
		Message secondMessage = new RequestMessage("second message");
		MessageQueue queue = new MessageQueue();
		assertNull(queue.front());
		queue.push(firstMessage);
		assertTrue(queue.front()==firstMessage);
		queue.push(secondMessage);
		assertTrue(queue.front()==firstMessage);
		queue.pop();
		assertTrue(queue.front()==secondMessage);
		queue.pop();
		assertNull(queue.front());
	}
	
	@Test
	public void testPop() {
		Message firstMessage = new RequestMessage("first message");
		Message secondMessage = new RequestMessage("second message");
		MessageQueue queue = new MessageQueue();
		queue.addObserver(this);
		queue.pop();
		assertEquals(0, queue.size());
		queue.push(firstMessage);
		queue.push(secondMessage);
		queue.pop();
		assertEquals(1, queue.size());
		assertTrue(this.message==secondMessage);
		assertTrue(queue.front()==secondMessage);
		queue.pop();
		assertEquals(0, queue.size());
		assertTrue(this.message==secondMessage);
		assertNull(queue.front());
		queue.deleteObserver(this);
	}
	
	@Test
	public void testReset() {
		MessageQueue queue = new MessageQueue();
		Message firstMessage = new RequestMessage("first message");
		Message secondMessage = new RequestMessage("second message");
		queue.push(firstMessage);
		queue.push(secondMessage);
		assertEquals(2, queue.size());
		queue.reset();
		assertEquals(0, queue.size());
	}
	
	@Test
	public void testSave() {
		String filename = "testQueue.seq";
		MessageQueue queue = new MessageQueue();
		Message firstMessage = new RequestMessage("first message");
		Message secondMessage = new RequestMessage("second message");
		queue.push(firstMessage);
		queue.push(secondMessage);
		queue.saveAs(filename);
		assertTrue(new File(filename).exists());
		MessageQueue newQueue = new MessageQueue();
		newQueue.load(filename);
		assertTrue(newQueue.size()==2);
		assertTrue(newQueue.front().getData().equals("first message"));
		newQueue.pop();
		assertTrue(newQueue.front().getData().equals("second message"));
		assertTrue(new File(filename).delete());
	}
	
	@Override
	public void update(Observable sender, Object aspect) {
		if (aspect.equals(MessageQueue.Aspect.CONTENT)) return;
		this.message = (Message)aspect;	
	}
}
